% ********************************************************************
% This script creates a simple visualization of the pendulum and vessel.
% ********************************************************************
%
% ***********************************
% Author: Rotem Sharoni
% Last updated: 29th of February 2016
% ***********************************
%
% The script uses y as the output from a Simulink simulation, where y is
% y.time and y.data(:,1) is angle of pend. and y.data(:,2) is x position

%Time depending values for X and theta (input)
XPOS=y.data(:,2)'; %x position of the vessel
theta=y.data(:,1)'; %Pendulum angle

close all

% Initiallize plots and creating a handle to graphic object
h1 = plot(NaN,NaN,'k','Linewidth',2); 
hold on
h2 = plot(NaN,NaN,'r'); 
h3 = plot(NaN,NaN,'r'); 
h4 = plot(NaN,NaN,'r'); 
h5 = plot(NaN,NaN,'r'); 
h6 = plot(NaN,NaN,'r'); 
h7 = plot(NaN,NaN,'b','Linewidth',2); 
h8 = plot(NaN,NaN,'b','Linewidth',2); 

time=y.time; %Simulation time (from Simulink)
rbottom=0.398/2; %Radius of bottom of vessel
h=0.129; %Height og vessel
rtop=0.598/2; %Radius of bottom of vessel
rbox=0.02; %Radius of semi cirle (pendulum hinge)
xval=linspace(0,pi,50); %Creating points to draw a semi circle 
BoxX=rbox*cos(xval)*2; %Semi circle (pendulum hinge)
BoxY=rbox*sin(xval)*2; %Semi circle (pendulum hinge)
Lpendulum=1.5; %Length of pendulum (m)

for i=1:length(time)
    pause(0.01); %Step size in Simulink
    
    %Updates the graphs width data of vessel and pendulum
    set(h1, 'XData', [XPOS(i)+0 XPOS(i)+Lpendulum*sin(theta(i))] ,...
        'YData', [0+h/2+rbox rbox+h+Lpendulum*cos(theta(i))]);
    set(h2, 'XData', [-rbottom+XPOS(i) rbottom+XPOS(i)] , 'YData',...
        [-h/2 -h/2]);
    set(h3,'XData',[-rtop+XPOS(i) rtop+XPOS(i)],'YData', [h/2 h/2])
    set(h4,'XData',[-rbottom+XPOS(i) -rtop+XPOS(i)],'YData',[-h/2 h/2])
    set(h5,'XData',[rbottom+XPOS(i) rtop+XPOS(i)],'YData',[-h/2 h/2])
    set(h6,'XData',[XPOS(i)+BoxX],'YData',[BoxY+h/2])
    set(h7,'XData',[-20 XPOS(i)-(rbottom+rtop)/2],'YData',[0 0])
    set(h8,'XData',[XPOS(i)+(rbottom+rtop)/2 20],'YData',[0 0])
    
    title([['Time elapsed:  '] [num2str(time(i),'%5.2f')] [' seconds']]);
    
    %Axis limits
    xlim([-3.5 3.5]); 
    ylim([-0.5 2]);
end


    