% ********************************************************************
% This script initializes parameters and runs the MIP model with surge and
% pitch coupling, as well as hydrodynamic forces.
% ********************************************************************
%
% ***********************************
% Author: Rotem Sharoni
% Last updated: 10th of May 2016
% ***********************************
%

% Initial parameters: 
x_init = [0 deg2rad(0) deg2rad(10)]'; %Initial "pose" of the MIP
rt = 0.135; % Radius from CG to motor axis in m
Tr=[1 1 1;
    0 0 0;
    0 rt -rt]; %Thrust allocation
pinvTr = pinv(Tr); % Pseudoinverse

% Plotting and simulation: 
tsim = 10; %Simulation time, s
sim MIPv2_hydro

%Plot the results:
set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

figure %Plot the state and velocities
subplot(3,1,1)
plot(x.time,x.data(:,1))
hold on 
plot(x_d.time,x_d.data(:,1));
legend('$x$ [m]','$\dot{x}$ [m/s]')
xlabel('Time (s)')
% title('Position')
subplot(3,1,2)
plot(x.time,rad2deg(x.data(:,2)))
hold on 
plot(x_d.time,rad2deg(x_d.data(:,2)));
legend('$\theta_0$ [deg]','$\dot{\theta}_0$ [deg/s]')
xlabel('Time (s)')
% title('Pitch [deg]')
subplot(3,1,3)
plot(x.time,rad2deg(x.data(:,3)))
hold on 
plot(x_d.time,rad2deg(x_d.data(:,2)));
legend('$\theta_1$ [deg]' ,'$\dot{\theta}_1$ [deg/s]')
xlabel('Time (s)')
% title('Pendulum angle [deg]')

figure %Plot the forces and moments
subplot(3,1,1)
plot(tau.time,tau.data)
legend('X','M')
title('Thruster forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')
subplot(3,1,2)
plot(Fhydro.time,Fhydro.data)
legend('"x"','"$\theta_0$"','"$\theta_1$"')
title('Hydrodynamic forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')
subplot(3,1,3)
plot(Fpend.time,Fpend.data)
legend('"x"','"$\theta_0$"','"$\theta_1$"')
title('Pendulum forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')
